import os
from google_sheets import GoogleSheets
import fileutils
from project import Project


consumable = 'consumable'
non_consumable = 'non-consumable'


class ID:
    en = 0
    ru = 1


class Localisation(object):
    """docstring for Localisation"""

    supported = ['en-US', 'ru']

    def __init__(self):
        super(Localisation, self).__init__()
        self.id = ''
        self.title = ''
        self.description = ''


class Inapp(object):
    """docstring for Inapp"""

    def __init__(self):
        super(Inapp, self).__init__()
        self.id = ''
        self.name = ''
        self.cost = 0
        self.tier = 0
        self.type = consumable
        self.localisations = []
        self.screenshot = None

    def calculate_cost(self):
        self.cost = min(1000, self.cost)
        table = [1, 1], [2, 2], [3, 3], [4, 4], [5, 5], [6, 6], [7, 7], [8, 8], [9, 9], [10, 10], \
            [11, 11], [12, 12], [13, 13], [14, 14], [15, 15], [16, 16], [17, 17], [18, 18], [19, 19], \
            [20, 20], [21, 21], [22, 22], [23, 23], [24, 24], [25, 25], [26, 26], [27, 27], [28, 28], \
            [29, 29], [30, 30], [31, 31], [32, 32], [33, 33], [34, 34], [35, 35], [36, 36], [37, 37], \
            [38, 38], [39, 39], [40, 40], [41, 41], [42, 42], [43, 43], [44, 44], [45, 45], [46, 46], \
            [47, 47], [48, 48], [49, 49], [50, 50], [55, 51], [60, 52], [65, 53], [70, 54], [75, 55], \
            [80, 56], [85, 57], [90, 58], [95, 59], [100, 60], [110, 61], [120, 62], [125, 63], [130, 64], \
            [140, 65], [150, 66], [160, 67], [170, 68], [175, 69], [180, 70], [190, 71], [200, 72], [210, 73], \
            [220, 74], [230, 75], [240, 76], [250, 77], [300, 78], [350, 79], [400, 80], [450, 81], [500, 82], \
            [600, 83], [700, 84], [800, 85], [900, 86], [1000, 87]
        for pair in table:
            if self.cost <= pair[0]:
                self.tier = pair[1]
                return
        print 'Cannot define tier by cost [{}]'.format(self.cost)

    def validate(self):
        if not self.id:
            return False, 'id cannot by empty'
        if not self.name:
            return False, 'name cannot by empty'
        if self.cost == 0:
            return False, 'cost have to be 1+'
        if not self.localisations:
            return False, 'inapp have to have localisation'
        return True, 'Ok'


class App(object):
    """docstring for App"""

    def __init__(self, package, name, version):
        super(App, self).__init__()
        self.package = package
        self.ios_team = Project.instance.apple_team_id
        self.name = name
        self.version = version
        self.inapps = []

    def validate(self):
        result = True
        for iap in self.inapps:
            valid, msg = iap.validate()
            if not valid:
                print 'Inapp [{}] has issue: {}'.format(iap.id, msg)
                result = False

        i = 0
        while i < len(self.inapps) - 1:
            j = i + 1
            while j < len(self.inapps):
                if self.inapps[i].id == self.inapps[j].id:
                    print 'All ID\'s have to be unique. Please check inapps with id [{}]'.format(self.inapps[i].id)
                    result = False
                if self.inapps[i].name == self.inapps[j].name:
                    print 'All ID\'s have to be unique. Please check inapps with id [{}]'.format(self.inapps[i].id)
                    result = False
                j += 1
            i += 1
        return result


def get_param(name, header, row):
    if name in header:
        index = header.index(name)
        return row[index].strip()
    return None


def parse_google_doc(app, google_doc_id):
    gs = GoogleSheets(CLIENT_SECRET_FILE=Project.instance.gg_secret_file)
    gs.set_document(google_doc_id)
    raw = gs.read_range('inapps', 'A1', 'Z')
    header = raw[0]
    inapps = raw[1:]

    for row in inapps:
        iap = Inapp()
        iap.id = get_param('id', header, row)
        iap.name = get_param('Unical Name', header, row)
        iap.cost = int(get_param('Price (USA)', header, row))
        iap.type = consumable if get_param('Consumable', header, row).lower() == 'yes' else non_consumable
        iap.screenshot = get_param('image (optional)', header, row)
        if not iap.screenshot or not len(iap.screenshot):
            iap.screenshot = iap.id
        if not iap.screenshot.endswith('.png') and not iap.screenshot.endswith('.jpg'):
            iap.screenshot += '.jpg'
        for localisation in Localisation.supported:
            lang = Localisation()
            lang.title = get_param(localisation + ' (title)', header, row).encode('utf-8')
            lang.description = get_param(localisation + ' (description)', header, row).encode('utf-8')
            iap.localisations.append(lang)
        iap.calculate_cost()
        app.inapps.append(iap)


def create_itmsp(app, argparser):
    local_data = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
\t<key>adamId</key>
\t<real>{3}</real>
\t<key>addOnCount</key>
\t<real>0.0</real>
\t<key>bundleId</key>
\t<string>{0}</string>
\t<key>name</key>
\t<string>{1}</string>
\t<key>sku</key>
\t<string>{0}</string>
\t<key>type</key>
\t<string>iOS App</string>
\t<key>version</key>
\t<string>{2}</string>
</dict>
</plist>'''.format(app.package, app.name, app.version, Project.instance.apple_id)
    metadata = '''<?xml version="1.0" encoding="UTF-8"?>
<package xmlns="http://apple.com/itunes/importer" version="software5.2">
    <metadata_token>{2}</metadata_token>
    <provider>{0}</provider>
    <team_id>{0}</team_id>
    <software>
        <vendor_id>{1}</vendor_id>
        <software_metadata>
            <in_app_purchases>__inapps__
            </in_app_purchases>
        </software_metadata>
    </software>
</package>'''.format(app.ios_team, app.package, Project.instance.apple_inapps_token)
    pattern_inapp = '''
                <in_app_purchase>
                    <locales>
                        <locale name="en-US">
                            <title>{8}</title>
                            <description>{9}</description>
                        </locale>
                        <locale name="ru">
                            <title>{10}</title>
                            <description>{11}</description>
                        </locale>
                    </locales>
                    <review_screenshot>
                        <file_name>{4}</file_name>
                        <size>{5}</size>
                        <checksum type="md5">{6}</checksum>
                    </review_screenshot>
                    <product_id>{7}.{0}</product_id>
                    <reference_name>{1}</reference_name>
                    <type>{3}</type>
                    <products>
                        <product>
                            <cleared_for_sale>true</cleared_for_sale>
                            <intervals>
                                <interval>
                                    <start_date>2017-07-12</start_date>
                                    <wholesale_price_tier>{2}</wholesale_price_tier>
                                </interval>
                            </intervals>
                        </product>
                    </products>
                </in_app_purchase>'''
    inapps_str = ''
    kind = '' if not Project.instance.with_pro_version else '.' + argparser.app_kind
    inapps_itmsp_folder = 'inapps%s.itmsp' % kind
    for i in app.inapps:
        image = fileutils.root_dir + '/store/{}/{}'.format(inapps_itmsp_folder, i.screenshot)
        if not os.path.isfile(image):
            print 'Error: [{}] image [{}] not founded. Please check it'.format(i.id, image)
            exit(-1)
        image_size = fileutils.getSize(image)
        image_md5 = fileutils.getMd5File(image)
        inapps_str += pattern_inapp.format(i.id, i.name, i.tier, i.type, i.screenshot, image_size, image_md5,
                                           app.package,
                                           i.localisations[ID.en].title, i.localisations[ID.en].description,
                                           i.localisations[ID.ru].title, i.localisations[ID.ru].description,
                                           )

    metadata = metadata.replace('__inapps__', inapps_str)
    fileutils.write(fileutils.root_dir + '/store/%s/machine-local-data.xml' % inapps_itmsp_folder, local_data)
    fileutils.write(fileutils.root_dir + '/store/%s/metadata.xml' % inapps_itmsp_folder, metadata)


def create_csv_android(app, argparser):
    csv = 'Product ID,Published State,Purchase Type,Auto Translate,Locale; Title; Description,Auto Fill Prices,Price,Pricing Template ID'
    pattern = '{0}.{1},published,managed_by_android,false,en_US; {3}; {4}; ru_RU; {5}; {6}, true,{2},'

    for i in app.inapps:
        rate = 60 * 1000 * 1000
        diff = 1000 * 1000
        inapps_str = pattern.format(app.package, i.id, i.cost * rate - diff,
                                    i.localisations[ID.en].title, i.localisations[ID.en].description,
                                    i.localisations[ID.ru].title, i.localisations[ID.ru].description
                                    )
        csv += '\n' + inapps_str
    kind = '' if not Project.instance.with_pro_version else '.' + argparser.app_kind
    fileutils.write(fileutils.root_dir + '/store/android_inapps%s.csv' % kind, csv)


def run(app_package, app_name, app_version, google_spreedsheet_id, argparser):
    app = App(app_package, app_name, app_version)
    parse_google_doc(app, google_spreedsheet_id)
    app.validate()
    create_itmsp(app, argparser)
    create_csv_android(app, argparser)
    print 'Finished'



import xml.etree.ElementTree as ET
from google_sheets import GoogleSheets
from project import Project
import fileutils
import validator_localization
from validate import print_log


class Locale:

    def __init__(self):
        self.values = ['', '']


def convert_to_table():
    locales = {}
    keys = []

    for i, lang in enumerate(['en', 'ru']):
        filepath = '{}/Resources/lang/{}.xml'.format(fileutils.root_dir, lang)
        root = ET.parse(filepath).getroot()
        dict_ = root.find('dict')
        key = ''
        value = ''
        for child in dict_:
            if child.tag == 'key':
                key = child.text
            elif child.tag == 'string':
                value = child.text
                if key and value and len(key) and len(value):
                    if key not in locales:
                        locale = Locale()
                        locales[key] = locale
                    locales[key].values[i] = value
                    # print len(locales), len(locales[key].values), locales[key], key, value

                    if key not in keys:
                        keys.append(key)
                    key = ''
                    value = ''

    for key in keys:
        print key + '\t' + '\t'.join(locales[key].values)


def download(document_id):
    locales = []

    gs = GoogleSheets(CLIENT_SECRET_FILE=Project.instance.gg_secret_file)
    gs.set_document(document_id)
    raw = gs.read_range(gs.get_sheet_titles()[0], 'A1', 'Z')
    header = raw[0]
    data = raw[1:]
    index_id = header.index('ID')
    index_en = header.index('EN')
    index_ru = header.index('RU')
    for line in data:
        id = line[index_id]
        ru = line[index_ru]
        en = line[index_en]
        locale = Locale()
        locale.values = [en, ru]
        locales.append([id, locale])

    en = ''
    ru = ''
    for item in locales:
        id = item[0]
        locale = item[1]
        en += '        <key>{}</key><string>{}</string>\n'.format(id, locale.values[0].encode('utf-8'))
        ru += '        <key>{}</key><string>{}</string>\n'.format(id, locale.values[1].encode('utf-8'))
    en = '<plist version="1.0">\n    <dict>\n' + en + '    </dict>\n</plist>'
    ru = '<plist version="1.0">\n    <dict>\n' + ru + '    </dict>\n</plist>'
    fileutils.write('{}/Resources/lang/en.xml'.format(fileutils.root_dir), en)
    fileutils.write('{}/Resources/lang/ru.xml'.format(fileutils.root_dir), ru)
    print 'Download finished.\n'
    print_log(validator_localization, 'Validate localization:')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from slackclient import SlackClient
from google_sheets import GoogleSheets
from datetime import datetime
import time
import sys

msg_on_approve = '''```Approve: Поздравляем! Тестировщику понравился билд *{}* и он готов к отправке дальше.
@{}, пожалуйста позаботься о его дальнейшей судьбе и смени статус.
{}```'''

msg_on_disapprove = '''```Disapprove: Ой-ой. Тестировщикам что-то не понравилось в *{}*.
@{}, разберись с проблемой срочно!
{}```'''

msg_on_overdue_milestone = '''```Overdue Milestone: ВНИМАНИЕ! Коллеги, у вас вчера был Milestone *{}*. Задачи из него не выполнены.
@{}, срочно отдай отчёт по проекту и обсуди с командой дату нового milestone.
Описание milestone {}
{}```'''

msg_on_notify_milestone = '''```Напоминание по проекту *{}* Дата milestone: {}
Менеджер проекта @{}
Описание:
{}
```'''


class Slack:

    def __init__(self):
        self.token = 'xoxp-185515669264-199828799040-278419184210-9f16965b9abba9335abe137d67065540'
        self.user = 'USLACKBOT'
        self.slack_client = SlackClient(self.token)
        self.api_call = self.slack_client.api_call("im.list")

    def send_message(self, channel, message, as_bot=True):
        self.slack_client.api_call('chat.postMessage', link_names=1, channel=channel, text=message,
                                   as_user=not as_bot, icon_emoji=':robot_face:', username='The Better Manager')


class GoogleParser:

    def __init__(self, slack, debug_mode=False):
        self.slack = slack
        self.debug_mode = debug_mode
        self.gs = GoogleSheets(CLIENT_SECRET_FILE='google_drive_client_secret.json')
        self.gs.set_document('1RVZg7PoqMS0ON-fg1QjfnY5LRczQSS_jsxrZrCEu6lA')
        self.range_testing = self.gs.read_range('Testing Statuses', 'A1', 'L')
        self.range_milestones = self.gs.read_range('Milestones From Form', 'A1', 'L')
        self.range_common = self.gs.read_range('Milestones', 'A1', 'ZZ')
        self.managers = {}
        self.channels = {}
        self.parse_managers()

    def parse_managers(self):
        row_projects = self.range_common[6]
        row_channels = self.range_common[7]
        row_slacks = self.range_common[8]
        for i, project in enumerate(row_projects):
            if not project:
                continue
            self.channels[project] = row_channels[i]
            self.managers[project] = row_slacks[i]

    def get_slack_manager_of_project(self, project):
        return self.managers[project] if not self.debug_mode else 'volodar'

    def get_slack_channel_of_project(self, project):
        return '#' + self.channels[project] if not self.debug_mode else '@volodar'

    def get_slack_channel_general(self):
        return '#general' if not self.debug_mode else '@volodar'

    @staticmethod
    def get_header(range):
        header = range[0]
        range = range[1:]
        return header, range

    @staticmethod
    def get_value(header, row, name):
        index = header.index(name)
        return row[index]

    @staticmethod
    def convert_data(date):
        m, d, t = date.split('/')
        human = '{}.{}.{}'.format(d, m, t)
        return human

    def send_notification_testing_status(self, header, row, message):
        status = GoogleParser.get_value(header, row, 'Status')
        project = GoogleParser.get_value(header, row, 'Project Name')
        comment = GoogleParser.get_value(header, row, 'Comments')
        manager = self.get_slack_manager_of_project(project)
        channel = self.get_slack_channel_of_project(project)
        slack_message = msg_on_approve if status == 'Approve' else msg_on_disapprove
        slack_message = slack_message.format(project, manager, comment)
        self.slack.send_message(channel, slack_message)
        self.slack.send_message(self.get_slack_channel_general(), slack_message)

    def warning_testing_statuses(self):
        header, range = GoogleParser.get_header(self.range_testing)
        for row in range:
            project = GoogleParser.get_value(header, row, 'Project Name')
            if not project:
                continue
            status = GoogleParser.get_value(header, row, 'Status')
            if status == 'Approve':
                self.send_notification_testing_status(header, row, 'Send build to store/customer')
            elif status == 'Disapprove':
                self.send_notification_testing_status(header, row, 'SNeed dispatch disapprove')

    def warning_milestones(self):
        header, range = GoogleParser.get_header(self.range_milestones)
        for row in range:
            project = GoogleParser.get_value(header, row, 'Project')
            if not project:
                continue
            status = GoogleParser.get_value(header, row, 'Status')
            if status in ['Release', 'Done', 'Moved']:
                continue
            finish_date = GoogleParser.get_value(header, row, 'Date')
            finish_time = time.mktime(datetime.strptime(finish_date, "%m/%d/%Y").timetuple())
            now = datetime.now()
            now_date = '{}/{}/{}'.format(now.month, now.day, now.year)
            now_time = time.mktime(datetime.strptime(now_date, "%m/%d/%Y").timetuple())

            overdue = now_time > finish_time
            if overdue:
                manager = self.get_slack_manager_of_project(project)
                channel = self.get_slack_channel_of_project(project)
                comment = GoogleParser.get_value(header, row, 'Description').encode('utf-8')
                slack_message = msg_on_overdue_milestone
                slack_message = slack_message.format(project, GoogleParser.convert_data(finish_date), manager, comment)
                self.slack.send_message(channel, slack_message)
                self.slack.send_message(self.get_slack_channel_general(), slack_message)

    def notify_about_milestones(self):
        header, range = GoogleParser.get_header(self.range_milestones)
        for row in range:
            project = GoogleParser.get_value(header, row, 'Project')
            if not project:
                continue
            status = GoogleParser.get_value(header, row, 'Status')
            if status in ['Release', 'Done', 'Moved']:
                continue
            finish_date = GoogleParser.get_value(header, row, 'Date')
            manager = self.get_slack_manager_of_project(project)
            comment = GoogleParser.get_value(header, row, 'Description').encode('utf-8')
            channel = self.get_slack_channel_of_project(project)
            slack_message = msg_on_notify_milestone
            slack_message = slack_message.format(project, GoogleParser.convert_data(finish_date), manager, comment)
            self.slack.send_message(channel, slack_message)


def run(event, debug_mode=False):
    g = GoogleParser(Slack(), debug_mode)
    if event == 'hour':
        g.warning_testing_statuses()
        g.warning_milestones()
    elif event == 'daily':
        g.notify_about_milestones()

if __name__ == '__main__':
    if len(sys.argv) > 1:
        event = sys.argv[1]
    else:
        event = 'daily'
    run(event)



from __future__ import division
import os
import fileutils
import shutil
from project import Project


def clone():
    out_folder = '{}/proj.ios'.format(fileutils.root_dir)
    if os.path.isdir(out_folder):
        shutil.rmtree(out_folder)
    cmd = 'git clone git@bitbucket.org:stereo7_tools/template_proj_ios_syn.git {}'.format(out_folder)
    print cmd
    os.system(cmd)
    shutil.rmtree('{}/.git'.format(out_folder))
    os.remove('{}/.gitignore'.format(out_folder))


def get_project_folder(arg_parser):
    folder = fileutils.root_dir + '/proj.ios/'
    return folder


def replace_values(kind, file):
    if not os.path.isfile(file):
        return
    content = open(file).read()
    content = content.replace('@{PACKAGE_NAME}', Project.instance.package)
    content = content.replace('@{APP_BUNDLE_NAME}', Project.instance.app_bundle_name)
    if kind and Project.instance.with_pro_version:
        content = content.replace('@{FACEBOOK_APP_ID}', Project.instance.services['ios'][kind].facebook_id)
        content = content.replace('@{FLURRY_APP_ID}', Project.instance.services['ios'][kind].flurry_id)
        content = content.replace('@{APPODEAL_APP_ID}', Project.instance.services['ios'][kind].appodeal_id)
    else:
        content = content.replace('@{FACEBOOK_APP_ID}', Project.instance.services['ios'].facebook_id)
        content = content.replace('@{FLURRY_APP_ID}', Project.instance.services['ios'].flurry_id)
        content = content.replace('@{APPODEAL_APP_ID}', Project.instance.services['ios'].appodeal_id)

    fileutils.write(file, content)


def copy_resources(arg_parser):
    proj_folder = get_project_folder(arg_parser)
    icons_lite = '{}/store/icons/'.format(fileutils.root_dir)
    icons_pro = '{}/store/icons/pro/'.format(fileutils.root_dir)
    if Project.instance.with_pro_version:
        icons_lite += 'lite/'
    icons = [20, 29, 40, 58, 60, 76, 80, 87, 120, 152, 167, 180, 1024]
    for size in icons:
        if Project.instance.with_pro_version:
            shutil.copy2('{}Icon-{}.png'.format(icons_lite, size), '{}ios/Images.xcassets/AppIcon.lite.appiconset/{}.png'.format(proj_folder, size))
            shutil.copy2('{}Icon-{}.png'.format(icons_pro, size), '{}ios/Images.xcassets/AppIcon.pro.appiconset/{}.png'.format(proj_folder, size))
        else:
            shutil.copy2('{}Icon-{}.png'.format(icons_lite, size), '{}ios/Images.xcassets/AppIcon.appiconset/{}.png'.format(proj_folder, size))


def run(arg_parser):
    clone()
    replace_values('', get_project_folder(arg_parser) + 'Syndicate.xcodeproj/project.pbxproj')
    replace_values('lite', get_project_folder(arg_parser) + 'ios/info.lite.plist')
    replace_values('pro', get_project_folder(arg_parser) + 'ios/info.pro.plist')
    replace_values('', get_project_folder(arg_parser) + 'ios/info.plist')
    copy_resources(arg_parser)
